{DelphiSim by Roeland van der Spek           }
{spekw@acm.org                               }
{(C) 1997                                    }
{v.2.00                                      }
{Simple Machine component                    }

unit machine;

interface

uses random,DelphiSim,classes,windows,synchronization;

Type TJob = class;

     TMachineProcess = class;

     TContinue = procedure (Load:TObject;machine:TMachineProcess) of object;
     TOnarrival= procedure (Load:TObject;Machine:TMachineProcess) of Object;

     TMachineProcess = class(TSimProcess)
     private
       FOnarrival:TOnArrival;
       FContinue:TContinue;
       FBeforeProcess:TContinue;
       ActiveTime:extended;         {Total active time of the machine}
       JobQ:TQueue;                 {List containing the pending jobs}
       NotOperational:boolean;      {Set to true if the machine is out of order}
       processedJobs:Longint;       {The total of jobs processed}
       temp:TJob;
       JobsAvailable:TSimSignal;
     protected
       procedure body; override;
       {property ProcessBody;}
     Public
       procedure reset; override;
       constructor Create(Owner:TComponent);   override;
       destructor destroy; override;
       procedure process(O:TObject;T:Double);              {handles the job}
       function  jobsPending:longint;             {returns the number of pending jobs}
     published
       property jobsdone:longint read processedJobs;   {number of jobs performed}
       property TimeActive:Extended read ActiveTime;
       property Onarrival:TOnArrival read fonarrival write fonArrival;
       property BeforeProcessing:TContinue read FBeforeProcess write FBeforeProcess;
       property AfterProcessing:TContinue read FContinue write fContinue;
     end;

     Tjob = class(Tobject)
     public
       j:TObject;
       Time:Double;
       constructor Create(O:TObject;T:Double);
     end;

     procedure register;

implementation

Constructor Tjob.create(O:TObject;T:Double);
begin
  inherited create;
  J:=O; {J=the job}
  Time:= T;
end;

Constructor TMachineProcess.Create(Owner:TComponent);
begin
  inherited create(Owner);  {Machine is a simprocess}
  NotOperational:=False;
  JobQ:=TQueue.create;
  ActiveTime:=0;
  processedJobs:=0;
  JobsAvailable:=TSimSignal.create(self);
end;

function TMachineProcess.JobsPending:Longint;
begin
  result := JobQ.count;
end;

procedure TMachineProcess.process(O:TObject;T:double);
{put the job on the queue, if the machine is not working then the
 machine is activated}
var P:Tjob;
begin
  If assigned(FOnArrival) then FOnArrival(O,self);    {perform tasks on incomming object prior to insertion in the jobqueue}
  P:=TJob.create(O,T);
  JobQ.add(P);
  JobsAvailable.Signal; {Tell the machine that a job is waiting}
end;

Procedure TMachineProcess.Reset;
var I:Longint;
    P:TJob;
begin
  inherited reset;
  NotOperational:=False;
  for i:=0 to jobq.count -1  do
  begin
    P:=JOBQ.items[i];
    P.j.free;
    P.free;
  end;
  JobQ.clear;
  ActiveTime:=0;
  processedJobs:=0;
end;

destructor TMachineProcess.destroy;
var P:TJob;
    v:Tobject;
begin
  while not jobq.isempty do
  begin
    P:=JobQ.pop;
    P.j.free;
    P.free;
  end;
  jobq.free;
  inherited destroy;
end;



Procedure TMachineProcess.body;
var ActiveStartTime:extended;
    ActiveEndTime:extended;
    P:TJob;
begin
  while not false do
  begin
    while JobsPending >0 do
    begin
      P:=JobQ.pop;
      if assigned(FBeforeProcess) then FBeforeProcess(P.J,self);
      ActiveStartTime:=SimTime;
      holdfor(P.Time);                {hold machine... processing}
      ActiveEndTime:=Simtime;      {processing is ready}
      ActiveTime:=ActiveTime+ActiveEndTime-ActiveStartTime;
      processedJobs:=ProcessedJobs+1;
      If assigned(FContinue) then FContinue(P.J,self);    {Manipulated object after processing}
      p.free;
    end;
    WaitFor(JobsAvailable);
  end;
end;


procedure register;
begin
  RegisterComponents('DelphiSim', [TMachineProcess]);
end;

end.
