{DelphiSim by Roeland van der Spek           }
{spekw@acm.org                               }
{(C) 1997                                    }
{v2.00                                       }
{Tally Component                             }


unit Tally;

interface

Uses classes,DelphiSim,sysutils;

type

   TTallyItem = Class(TObject)
   public
     Replication:Longint;
     Time,value:double;
   end;

   TTally = class(TComponent)
   private
     List:TList;  {List of values}
     MaxVal,MinVal:double;
     sim:TSimulation;
   protected
     STime:Double;
   public
     Constructor Create(Owner:Tcomponent); override;
     destructor destroy; override;
     procedure reset;
     procedure Tally(value:Double);
     Function min:double;
     function Max:double;
     function avg:double;
     function stdev:double;
     function count:longint;
     function measurements:TStringlist;
   published
     property Simulation:TSimulation read sim write sim;
     property StartTime:Double read Stime write Stime;
   end;

   procedure register;

implementation

constructor TTally.create(Owner:TComponent);
begin
  inherited create(owner);
  Stime:=Starttime;
  List:=TList.create;
  MaxVal:=0;
  MinVal:=0;
end;

destructor TTally.destroy;
var i:Longint;
    P:TTallyItem;
begin
  for i:=0 to List.count -1 do
  begin
    P:=List.items[i];
    P.free;
  end;
  list.free;
  inherited destroy;
end;

procedure TTally.reset;
begin
  list.clear
end;

procedure TTally.tally(value:Double);
var P:TTallyItem;
begin
  if Sim.simtime >= stime then
  begin
    P:=TTallyItem.create;
    P.value:=value;
    P.time:=Sim.SimTime;
    P.Replication:=Sim.Replications-Sim.Repsleft;
    List.add(P);
    if value>maxval then maxval :=value;
    if value<minval then minval :=value;
  end;
end;

function TTally.max:double;
begin
  result:=maxval;
end;

Function TTally.measurements:TStringlist;
var L:TStringlist;
    i:longint;
    s:String;
    P:TTallyItem;
begin
  L:=TStringlist.create;
  for i:=0 to List.count do
  begin
    P:=list.items[i];
    s:=format('%d %f %f ',[P.replication,P.time,P.value]);
    L.add(s);
  end;
  result:=L;
end;

function TTally.min:double;
begin
  result:=minval;
end;

function TTally.avg:double;
var i:Longint;
    a:double;
    p:TTallyItem;
begin
  if list.count=0 then result:=0
  else
  begin
    a:=0;
    for i:=0 to List.count-1 do
    begin
      P:=List.items[i];
      a:=a+P.value;
    end;
    result:=a/list.count;
  end;
end;

function TTally.count:longint;
begin
  result:=List.count;
end;

function TTally.Stdev:double;
var P:TTallyItem;
    I:longint;
    temp,s:double;
    Tavg:double;
begin
  s:=0;
  if list.count < 2 then result:=0
  else
  begin
    Tavg:=avg;
    for i:=0 to list.count -1 do
    begin
      P:=list.items[i];
      temp:=(P.value-tavg);
      s:=s+temp*temp;
    end;
    result:=Sqrt(s/(List.count -1));
  end;
end;

procedure register;
begin
  registercomponents('DelphiSim',[TTally]);
end;

end.
